package com.anyway.favor.controller;

import com.anyway.favor.model.Person;
import com.anyway.favor.model.User;
import com.anyway.favor.model.enums.Update;
import com.anyway.favor.model.vo.PersonVo;
import com.anyway.favor.service.PersonService;
import com.anyway.util.PageQuery;
import com.anyway.util.Pair;
import com.anyway.util.R;
import com.anyway.util.SessionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.StringUtils;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotBlank;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 人员控制器
 *
 * @author liuchuanwei
 * @date 2024-02-20
 */
@RestController
@RequestMapping("/person")
public class PersonController {
    @Autowired
    private PersonService personService;

    /**
     * 添加人员
     *
     * @param person
     * @return
     */
    @ResponseBody
    @RequestMapping("/add")
    public R<Void> add(@Validated @RequestBody Person person) {
        User currentUser = SessionUtils.currentUser();
        person.setCreateBy(currentUser.getId());
        return personService.add(person) ? R.ok() : R.fail("新增失败");
    }
    /**
     * 更新人员
     *
     * @param person
     * @return
     */
    @PostMapping("/update")
    public R<Void> update(@Validated({Update.class}) @RequestBody Person person) {
        User currentUser = SessionUtils.currentUser();
        person.setModifyBy(currentUser.getId());
        return personService.update(person) ? R.ok() : R.fail("保存失败");
    }

    /**
     * 人员详情
     *
     * @param id
     * @return
     */
    @GetMapping("/detail/{id}")
    public R<PersonVo> detail(@PathVariable Long id) {
        Person person = personService.findById(id);
        return R.data(this.toPersonVo(person));
    }

    /**
     * 转为 PersonVo
     * @param person
     * @return
     */
    private PersonVo toPersonVo(Person person) {
        PersonVo personVo = new PersonVo(person);
        personVo.setMatePerson(personService.findById(person.getMateId()));
        personVo.setFatherPerson(personService.findById(person.getFatherId()));
        personVo.setMotherPerson(personService.findById(person.getMotherId()));
        return personVo;
    }

    /**
     * 人员列表
     *
     * @param pageQuery
     * @return
     */
    @ResponseBody
    @RequestMapping("/listPage")
    public R<List<PersonVo>> listPage(@RequestBody PageQuery<Map<String, Object>> pageQuery) {
        User currentUser = SessionUtils.currentUser();
        Map<String, Object> mapCondition = pageQuery.getTerms(new HashMap<>());
        //查询和当前登录用户相关的人员
        mapCondition.put("createBy", currentUser.getId());
        List<Person> personList = personService.findPage(pageQuery);
        //设置关联属性
        List<PersonVo> personVoList = new ArrayList<>();
        for (Person person : personList) {
            personVoList.add(this.toPersonVo(person));
        }
        return R.page(personVoList, pageQuery.getPage());
    }

    /**
     * 人员列表选项
     *
     * @return
     */
    @GetMapping("/listOption")
    public R<List<Pair<String, Long>>> listOption(@RequestBody PageQuery<Map<String, Object>> pageQuery) {
        User currentUser = SessionUtils.currentUser();
        Map<String, Object> mapCondition = pageQuery.getTerms(new HashMap<>());
        //查询和当前登录用户相关的人员
        mapCondition.put("createBy", currentUser.getId());
        List<Person> personList = personService.findPage(pageQuery);
        List<Pair<String, Long>> personOptionList = Pair.toList(personList, e -> e.getCallName() + "(" + e.getName() + ")", Person::getId);
        return R.data(personOptionList);
    }


    /**
     * 批量删除
     *
     * @param ids
     * @return
     */
    @PostMapping("/delete")
    public R<Void> delete(@RequestParam(defaultValue = "") @NotBlank(message = "ids不能为空") String ids) {
        if(!StringUtils.hasText(ids)) {
            return R.fail("无效的ID");
        }
        String[] arr = ids.split(",");
        List<Long> idList = Arrays.stream(arr).map(Long::valueOf).collect(Collectors.toList());
        int r = personService.deleteByIds(idList);
        return r > 0 ? R.ok() : R.fail("删除失败");
    }
}
