package com.anyway.favor.service.impl;

import com.anyway.exception.BusinessException;
import com.anyway.favor.dao.PersonDao;
import com.anyway.favor.model.Person;
import com.anyway.favor.service.PersonService;
import com.anyway.util.PageQuery;
import com.anyway.util.PageUtils;
import com.google.common.cache.CacheBuilder;
import com.google.common.cache.CacheLoader;
import com.google.common.cache.LoadingCache;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Nullable;
import javax.annotation.PostConstruct;
import java.util.Arrays;
import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 人员业务实现类
 *
 * @author anyway
 * @date 2024-02-20
 */
@Slf4j
@Service
public class PersonServiceImpl implements PersonService {

    @Autowired
    private PersonDao personDao;
    /**
     * 人员缓存
     */
    private LoadingCache<Long, Person> personCache;

    @PostConstruct
    public void init() {
        personCache = CacheBuilder.newBuilder()
                .maximumSize(100)
                .expireAfterWrite(5, TimeUnit.MINUTES)
                .build(new CacheLoader<Long, Person>() {
                    @Override
                    public Person load(Long id) {
                        return personDao.findById(id);
                    }
                });
    }

    @Override
    public List<Person> findAll() {
        return personDao.findAll();
    }

    @Override
    public List<Person> findByCondition(Map<String, Object> map) {
        return personDao.findByCondition(map);
    }

    @Override
    public List<Person> findPage(PageQuery<Map<String, Object>> pageQuery) {
        PageUtils.startPage(pageQuery.getPage());
        List<Person> personList = personDao.findByCondition(pageQuery.getTerms());
        PageUtils.setPageTotal(personList, pageQuery.getPage());
        return personList;
    }

    @Override
    @Transactional
    public boolean add(Person person) {
        this.setAssociatedProperty(person);
        int c = personDao.add(person);
        this.updateMate(person);
        return c > 0;
    }

    /**
     * 设置关联属性
     * @param p
     */
    private void setAssociatedProperty(Person p) {
        //如果配偶ID不为空，则设置为已婚
        if (p.getMateId() != null) {
            p.setMaritalStatus(1);
        }
    }

    @Transactional
    @Override
    public boolean update(Person person) {
        this.setAssociatedProperty(person);
        this.updateMate(person);
        return personDao.update(person) > 0;
    }

    @Override
    public Person findById(@Nullable Long id) {
        if (id != null) {
            return personCache.getUnchecked(id);
        }
        return null;
    }

    @Transactional
    @Override
    public int delete(Long id) {
        int c = personDao.deleteByIds(Arrays.asList(id));
        if (c < 1) {
            return c;
        }
        //更新相关人员的配偶、父母
        personDao.updateNullMate(id);
        personDao.updateNullMother(id);
        personDao.updateNullFather(id);
        return c;
    }

    @Transactional
    @Override
    public int deleteByIds(Collection<Long> ids) {
        for (Long id : ids) {
            this.delete(id);
        }
        return 1;
    }

    /**
     * 更新配偶人员
     *
     * @param person
     */
    private void updateMate(Person person) {
        if (person.getMateId() == null) {
            //解除婚姻状态
            if (person.getId() != null) {
                log.info("解除配偶关系:{}", person.getId());
                personDao.updateNullMate(person.getId());
                person.setMaritalStatus(0);
            }
            return ;
        }
        //配偶人员
        Person matePerson = this.findById(person.getMateId());
        if (matePerson == null) {
            log.error("配偶人员{}不存在", person.getMateId());
            throw new BusinessException("无效的配偶");
        }

        if(matePerson.getMateId() == null) {
            log.info("关联配偶关系:{}, {}", person.getMateId(), person.getId());
            matePerson.setMateId(person.getId());
            matePerson.setMaritalStatus(1);
            personDao.update(matePerson);
        } else if (!person.getMateId().equals(matePerson.getMateId())) {
            log.error("该配偶{}的配偶{}不是当前人员", person.getMateId(), matePerson.getMateId());
            throw new BusinessException("该配偶的配偶不是当前人员");
        }
    }
}
